// Generated by CoffeeScript 1.10.0
var args, bits, cmd, compile, compiler, compilers, conf, confs, credentials, cwd, defaultExclude, err, error, error1, error2, exec, ext, firstAddress, fs, i, j, len, len1, log, mod, opt, opts, os, p, ref, ref1, ref2, ref3, ref4, relativeName, requireConfig, server, usage, version, vigil, vigilOpts;

exec = require('executive');

fs = require('fs');

os = require('os');

vigil = require('vigil');

compilers = require('./compilers');

log = require('./log');

server = require('./server');

ref = require('./utils'), defaultExclude = ref.defaultExclude, firstAddress = ref.firstAddress;

error = function(message) {
  log.error(message);
  return process.exit(1);
};

version = function() {
  console.log((require('../package.json')).version);
  return process.exit(0);
};

usage = function() {
  console.log("bebop <command> [options] [file]\n\nCommands:\n  compile|c  Compile files and exit\n  help       Display this message\n  version    Display version\n\nOptions:\n  --auto                      Automatically compile even without a local config file\n  --compilers <ext:compiler>  Specify compiler to use for a given extension\n  --config <file>             Specify bebop.coffee to use\n  --exclude, -x <file>        Exclude files from watching, compiling\n  --force-reload              Force reload when file is compiled\n  --host, -h <hostname>       Hostname to bind to\n  --include, -i <file>        Include files for watching, compiling\n  --no-server                 Do not run static file server\n  --no-watch                  Do not watch files for changes\n  --open, -o                  Open browser automatically\n  --port, -p <port>           Port to listen on\n  --pre <cmd>                 Command to execute first\n  --secure, -s <user:pass>    Require authentication\n  --asset-dir <path>          Directory used as root for compiling, watching\n  --build-dir <path>          Directory used as root for static file server\n  --work-dir  <path>          Directory used as root for process\n  --index <file>              Index file to attempt to serve when directory requested");
  return process.exit(0);
};

try {
  require('coffee-script/register');
} catch (error1) {
  err = error1;
}

cwd = process.cwd();

confs = [process.env.HOME + '/.bebop', cwd + '/.bebop', cwd + '/bebop'];

opts = {
  compile: false,
  compileOnly: false,
  compilers: {},
  defaultExclude: true,
  exclude: [],
  fallback: null,
  forceReload: false,
  host: firstAddress(),
  include: [],
  index: ['index.html', 'index.htm'],
  initialPath: '',
  port: null,
  pre: function(done) {
    return done();
  },
  runServer: true,
  watch: true,
  assetDir: cwd,
  buildDir: cwd,
  workDir: cwd
};

requireConfig = function(path) {
  var conf, error2, k, ref1, v;
  try {
    conf = require.resolve(path);
  } catch (error2) {
    err = error2;
    return;
  }
  if (fs.existsSync(conf)) {
    ref1 = require(conf);
    for (k in ref1) {
      v = ref1[k];
      opts[k] = v;
    }
    return opts.compile = true;
  }
};

for (i = 0, len = confs.length; i < len; i++) {
  conf = confs[i];
  requireConfig(conf);
}

args = process.argv.slice(2);

while (opt = args.shift()) {
  switch (opt) {
    case 'compile':
    case 'c':
    case '--compile':
    case '-c':
      opts.compileOnly = true;
      break;
    case 'help':
    case '--help':
      usage();
      break;
    case 'version':
    case '--version':
    case '-v':
      version();
      break;
    case '--config':
      requireConfig(args.shift());
      break;
    case '--open':
    case '-o':
      opts.open = true;
      break;
    case '--no-server':
      opts.runServer = false;
      break;
    case '--no-watch':
      opts.watch = false;
      break;
    case '--auto':
      opts.compile = true;
      break;
    case '--include':
    case '-i':
      opts.include.push(args.shift());
      break;
    case '--exclude':
    case '-x':
      opts.exclude.push(args.shift());
      break;
    case '--no-default-exclude':
      opts.defaultExclude = false;
      break;
    case '--force-reload':
      opts.forceReload = true;
      break;
    case '--fallback':
      opts.fallback = args.shift();
      break;
    case '--host':
    case '-h':
      opts.host = args.shift();
      break;
    case '--pre':
      cmd = args.shift();
      opts.pre = function(done) {
        return exec(cmd, done);
      };
      break;
    case '--port':
    case '-p':
      p = args.shift();
      if (!p) {
        error('missing port');
      }
      opts.port = parseInt(p, 10);
      break;
    case '--secure':
    case '-s':
      credentials = args.shift();
      if (credentials) {
        ref1 = credentials.split(':'), opts.user = ref1[0], opts.pass = ref1[1];
      } else {
        opts.user = 'bebop';
        opts.pass = 'beepboop';
      }
      break;
    case '--asset-dir':
      opts.assetDir = args.shift();
      break;
    case '--build-dir':
      opts.buildDir = args.shift();
      break;
    case '--work-dir':
      opts.workDir = args.shift();
      break;
    case '--compilers':
    case '-c':
      ref2 = args.shift().split(',');
      for (j = 0, len1 = ref2.length; j < len1; j++) {
        compiler = ref2[j];
        ref3 = compiler.split(':'), ext = ref3[0], mod = ref3[1];
        opts.compilers[ext] = mod;
      }
      break;
    default:
      if (opt.charAt(0) === '-') {
        error("Unrecognized option: '" + opt + "'");
      } else {
        opts.initialPath = opt;
      }
  }
}

if (opts.defaultExclude) {
  opts.exclude = [vigil.utils.excludeRe, defaultExclude].concat(opts.exclude);
}

vigilOpts = {
  exclude: opts.exclude,
  include: opts.include,
  patch: false
};

ref4 = opts.compilers;
for (ext in ref4) {
  compiler = ref4[ext];
  if (typeof compiler === 'string') {
    try {
      bits = compiler.split('.');
      compiler = require(bits.shift());
      while (bits.length) {
        compiler = compiler[bits.shift()];
      }
      compilers[ext] = compiler;
    } catch (error2) {
      err = error2;
      console.log(err);
    }
  } else {
    compilers[ext] = compiler;
  }
}

relativeName = function(filename) {
  return filename.replace(opts.workDir + '/', '');
};

compile = function(filename, cb) {
  if (cb == null) {
    cb = function() {};
  }
  return compilers.compile(filename, opts, function(err, compiled) {
    filename = relativeName(filename);
    if (err != null) {
      log.error("failed to compile " + filename, err);
      return;
    }
    if (compiled) {
      log.compiled(filename);
    }
    return cb(null, compiled);
  });
};

opts.pre(function(err) {
  var app, websocket;
  if (err != null) {
    return;
  }
  if (opts.compile) {
    vigil.walk(opts.workDir, vigilOpts, function(filename) {
      if (opts.compile) {
        return compile(filename);
      }
    });
  }
  if (opts.compileOnly) {
    return;
  }
  if (opts.runServer) {
    app = server.createServer(opts);
    websocket = (require('./websocket'))({
      server: app
    });
  } else {
    app = {
      run: function() {}
    };
    websocket = {
      modified: function() {}
    };
  }
  if (opts.watch) {
    vigil.watch(opts.assetDir, vigilOpts, function(filename) {
      if (!opts.compile) {
        log.modified(filename);
        return websocket.modified(filename);
      }
      return compile(filename, function(err, compiled) {
        if (!compiled) {
          log.modified(filename);
          return websocket.modified(filename);
        } else {
          if (opts.forceReload) {
            return websocket.modified(filename);
          }
        }
      });
    });
    if (opts.buildDir !== opts.assetDir) {
      vigil.watch(opts.buildDir, vigilOpts, function(filename) {
        log.modified(filename);
        return websocket.modified(filename);
      });
    }
  }
  return app.run(function() {
    if (opts.open || opts.initialPath !== '') {
      switch (os.platform()) {
        case 'darwin':
          return exec("open http://" + opts.host + ":" + opts.port + "/" + opts.initialPath);
        case 'linux':
          return exec("xdg-open http://" + opts.host + ":" + opts.port + "/" + opts.initialPath);
      }
    }
  });
});
