// Generated by CoffeeScript 1.10.0
var basicAuth, connect, fakeExpress, favicons, firstAddress, log, logger, markdown, middleware, path, serveIndex, serveStatic, stripHtml, stripSlash, trailingHtmlRe, trailingSlashRe, url;

basicAuth = require('basic-auth-connect');

connect = require('connect');

favicons = require('connect-favicons');

logger = require('morgan');

path = require('path');

serveIndex = require('serve-index');

serveStatic = require('serve-static');

url = require('url');

log = require('./log');

markdown = require('./markdown');

middleware = require('./middleware');

firstAddress = require('./utils').firstAddress;

trailingHtmlRe = /\.html$/;

trailingSlashRe = /\.html\/$/;

fakeExpress = function(req, res, next) {
  res.redirect = function(loc) {
    res.writeHead(302, {
      Location: loc
    });
    return res.end();
  };
  res.set = function(headers) {
    var k, results, v;
    results = [];
    for (k in headers) {
      v = headers[k];
      results.push(res.setHeader(k, v));
    }
    return results;
  };
  res.send = function(body) {
    return res.end(body);
  };
  url = url.parse(req.url);
  req.path = url.pathname;
  req.search = url.search;
  return next();
};

stripHtml = function(req, res, next) {
  var loc;
  if (!trailingHtmlRe.test(req.url)) {
    return next();
  }
  loc = req.url.replace(/index.html$/, '');
  loc = loc.replace(trailingHtmlRe, '');
  return res.redirect(loc);
};

stripSlash = function(req, res, next) {
  var loc;
  if (!trailingSlashRe.test(req.url)) {
    return next();
  }
  loc = req.url.replace(trailingSlashRe, '.html');
  return res.redirect(loc);
};

module.exports = {
  createServer: function(opts) {
    var app, dir, i, len, ref, ref1, ref2, ref3, serveOpts, server;
    if (opts == null) {
      opts = {};
    }
    if (opts.host == null) {
      opts.host = '0.0.0.0';
    }
    if (opts.port == null) {
      opts.port = 1987;
    }
    if (opts.buildDir == null) {
      opts.buildDir = process.cwd();
    }
    if (opts.workDir == null) {
      opts.workDir = process.cwd();
    }
    app = connect();
    app.use(fakeExpress);
    app.use(stripHtml);
    app.use(stripSlash);
    app.use(favicons(__dirname + '/../assets'));
    app.use(logger('dev'));
    if (opts.user && opts.pass) {
      app.use(basicAuth(opts.user, opts.pass));
    }
    app.use(middleware());
    app.use(markdown());
    serveOpts = {
      etag: false,
      fallthrough: true,
      dotfiles: (ref = opts.dotfiles) != null ? ref : 'deny',
      extensions: (ref1 = opts.extensions) != null ? ref1 : ['html', 'htm'],
      index: (ref2 = opts.index) != null ? ref2 : ['index.html', 'index.htm']
    };
    app.use(serveStatic(opts.buildDir, serveOpts));
    app.use(serveIndex(opts.buildDir, {
      hidden: true
    }));
    ref3 = [opts.assetDir, opts.workDir];
    for (i = 0, len = ref3.length; i < len; i++) {
      dir = ref3[i];
      if ((dir != null) && dir !== '' && dir !== opts.buildDir) {
        app.use(serveStatic(dir, serveOpts));
      }
    }
    server = require('http').createServer(app);
    server.setMaxListeners(100);
    server.on('listening', function() {
      return log.bebop("serving " + (path.basename(opts.workDir)) + " at http://" + opts.host + ":" + opts.port);
    });
    server.run = function(cb) {
      if (cb == null) {
        cb = function() {};
      }
      process.once('uncaughtException', function(err) {
        if (err.code === 'EADDRINUSE') {
          log.error('address in use, retrying...');
          server.close();
          opts.port++;
          return setTimeout(server.run, 1000);
        } else {
          log.error(err);
          return process.exit(1);
        }
      });
      return server.listen(opts.port, opts.host, cb);
    };
    return server;
  }
};
